import json
from pathlib import Path
from time import localtime,strftime

def show_response_card(gs_context,current_context,response_details):
    channel = gs_context['channel']
    if response_details['card_type'] == 'list':
        validate_list(channel,response_details)
        list_json = render_list(channel,response_details)
        current_context['response_queue'].append(list_json)
    elif response_details['card_type'] == 'quick_reply':
        validate_quick_reply(channel,response_details)
        quick_reply_json = render_quick_reply(channel,response_details)
        current_context['response_queue'].append(quick_reply_json)
    elif response_details['card_type'] == 'text':
        text = response_details['message']
        current_context['response_queue'].append(str(text))
    elif response_details['card_type'] == 'file':
        file_json = render_file(channel,response_details)
        current_context['response_queue'].append(file_json) 
    elif response_details['card_type'] == 'image':
        img_json = render_image(channel,response_details)
        current_context['response_queue'].append(img_json)                
    return

def validate_list(channel,response_details):
    if channel == 'whatsapp':
        len_title = len(response_details['main_title'])
        if len_title > 60:
            raise Exception("main_title has exceeded the character limit of 60!")
        len_body = len(response_details['main_body'])
        if len_body > 1024:
            raise Exception("main_body has exceeded the character limit of 1024!")
        len_button = len(response_details['button_text'])
        if len_button > 20:
            raise Exception("button_text has exceeded the character limit of 20!")
        for section in response_details['sections']:
            if 'title' not in section.keys():
                raise Exception("Section does not have a Title!")
            for option in section['options']:
                len_opt_title = len(option['title'])
                if len_opt_title > 24:
                    raise Exception(f"'{option['title']}' has exceeded the character limit of 24!")
                if 'description' in option.keys():
                    len_desc = len(option['description'])
                    if len_desc > 72:
                        raise Exception(f"'{option['description']}' has exceeded the character limit of 72!")
    return

def render_list(channel,response_details):
    list_json = {}
    if channel == 'whatsapp':
        main_title = response_details['main_title']
        main_body = response_details['main_body']
        button_text = response_details['button_text']
        section_list = response_details['sections']
        item_list = []
        for section in section_list:
            section_body = {}
            section_body['title'] = section['title']
            elements = section['options']
            options_list = []
            for element in elements:
                options = {}
                options['type'] = 'text'
                options['title'] = element['title']
                if 'description' in element.keys():
                    options['description'] = element['description']
                if 'postbackText' in element.keys():
                    options['postbackText'] = element['postbackText']
                options_list.append(options)
            section_body['options'] = options_list
            item_list.append(section_body)
        list_json['type'] = 'list'
        list_json['title'] = main_title
        list_json['body'] = main_body
        list_json['msgid'] = 'lst1'
        list_json['globalButtons'] = []
        list_json['globalButtons'].append({"type":"text","title":str(button_text)})
        list_json['items'] = item_list
    return list_json

def validate_quick_reply(channel,response_details):
    if channel == 'whatsapp':
        if 'header' in response_details.keys():
            len_header = len(response_details['header'])
            if len_header > 60:
                raise Exception("header has exceeded the character limit of 60!")
        if 'caption' in response_details.keys():
            len_caption = len(response_details['caption'])
            if len_caption > 60:
                raise Exception("cation has exceeded the character limit of 60!")
        len_options = len(response_details['options'])
        if len_options > 3:
            raise Exception("You cannot use more than 3 quick reply buttons in whatsapp!")
    return

def render_quick_reply(channel,response_details):
    quick_reply_json = {}
    if channel == 'whatsapp':
        buttons = response_details['options']
        options_list = []
        for button in buttons:
            button_dict = {}
            button_dict['type'] = 'text'
            button_dict['title'] = str(button)
            options_list.append(button_dict)
        quick_reply_json['type'] = 'quick_reply'
        quick_reply_json['content'] = {}
        quick_reply_json['content']['type'] = 'text'
        if 'header' in response_details.keys():
            quick_reply_json['content']['header'] = response_details['header']
        quick_reply_json['content']['text'] = response_details['body']
        if 'caption' in response_details.keys():
            quick_reply_json['content']['caption'] = response_details['caption']
        quick_reply_json['msgid'] = 'qr1'
        quick_reply_json['options'] = options_list
    return quick_reply_json

def render_file(channel,response_details):
    file_json = {}
    if channel == 'whatsapp':
        file_json['type'] = 'file'
        file_json['url'] = response_details['url']
        file_json['filename'] = response_details['filename']
    return file_json

def render_image(channel,response_details):
    image_json = {}
    if channel == 'whatsapp':
        image_json['type'] = 'image'
        image_json['originalUrl'] = response_details['url']
        image_json['previewUrl'] = response_details['url']
        image_json['caption'] = response_details['caption']
    return image_json

def behave_extract_bot_response_text(channel,bot_response):
    if channel == 'whatsapp':
        bot_text = ''
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] != '{':
                    bot_text = resp['text']
                    break
        else:
            bot_text = bot_response[0]['text']
        return bot_text

def behave_compare_text(input_text,bot_text):
    if input_text == bot_text:
        return True
    return False

def behave_extract_bot_response_quick_reply(channel,bot_response):
    if channel == 'whatsapp':
        bot_text = ''
        bot_buttons = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    qr_json = json.loads(resp['text'])
                    bot_text = qr_json['content']['text']
                    for option in qr_json['options']:
                        bot_buttons.append(option['title'])  
                    break
        else:
            qr_json = json.loads(bot_response[0]['text'])
            bot_text = qr_json['content']['text']
            for option in qr_json['options']:
                bot_buttons.append(option['title'])
        return bot_text.strip(),bot_buttons.sort()  

def behave_compare_quick_reply(input_message,bot_message,button_input,bot_buttons):
    if (input_message == bot_message) and (button_input == bot_buttons):
        return True
    return False

def behave_extract_bot_response_list(channel,bot_response):
    if channel == 'whatsapp':
        bot_text = ''
        bot_list = []
        if len(bot_response) > 1:
            for resp in bot_response:
                if resp['text'][0] == '{':
                    list_json = json.loads(resp['text'])
                    bot_text = list_json['body']
                    for item in list_json['items']:
                        list_element = {}
                        list_element["Category"] = item['title']
                        list_element["Items"] = []
                        for option in item['options']:
                            list_element["Items"].append(option['title'])
                        bot_list.append(list_element)
                    break
        else:
            list_json = json.loads(bot_response[0]['text'])
            bot_text = list_json['body']
            for item in list_json['items']:
                list_element = {}
                list_element["Category"] = item['title']
                list_element["Items"] = []
                for option in item['options']:
                    list_element["Items"].append(option['title'])
                bot_list.append(list_element)
        return bot_text.strip(),bot_list          

def behave_compare_list(input_message,bot_message,list_input,bot_list):      
    list_comparison = True
    for category in list_input:
        category['Items'].sort()
    for category in bot_list:
        category['Items'].sort()
    for element in list_input:
        if element not in bot_list:
            list_comparison = False
    for element in bot_list:
        if element not in list_input:
            list_comparison = False
    if (input_message == bot_message) and (list_comparison == True):
        return True
    return False

def log_behave_failures(card_type,input_message,bot_message,input_features,output_features):
    log_location = '/tmp/behave_failed.log'
    myfile = Path(log_location)
    myfile.touch(exist_ok=True)
    #timenow=strftime("%Y-%m-%d %H:%M:%S", localtime())
    if card_type == 'text':           
        with open(log_location, "a") as file_object:
            file_object.write("\n{0} - {1} validation failed!\n".format(str(strftime("%Y-%m-%d %H:%M:%S", localtime())),str(card_type)))
            file_object.write("Input/Feature Text:\n")
            file_object.write(input_message)
            file_object.write("\nOutput/Bot Text:\n")
            file_object.write(bot_message)
    if card_type == 'list':           
        with open(log_location, "a") as file_object:
            file_object.write("\n{0} - {1} validation failed!\n".format(str(strftime("%Y-%m-%d %H:%M:%S", localtime())),str(card_type)))
            file_object.write("Input/Feature Text:\n")
            file_object.write(input_message)
            file_object.write("\nInput/Feature Options:\n")
            file_object.write(str(input_features))
            file_object.write("\nOutput/Bot Text:\n")
            file_object.write(bot_message)
            file_object.write("\nOutput/Bot Options:\n")
            file_object.write(str(output_features))
    if card_type == 'quick_reply':           
        with open(log_location, "a") as file_object:
            file_object.write("\n{0} - {1} validation failed!\n".format(str(strftime("%Y-%m-%d %H:%M:%S", localtime())),str(card_type)))
            file_object.write("Input/Feature Text:\n")
            file_object.write(input_message)
            file_object.write("\nInput/Feature Options:\n")
            file_object.write(str(input_features))
            file_object.write("\nOutput/Bot Text:\n")
            file_object.write(bot_message)
            file_object.write("\nOutput/Bot Options:\n")
            file_object.write(str(output_features))
    return



from .response_cards import show_response_card

def format_response_card_quick_reply(gs_context, current_context, response):
    response_json_quick = {}
    response_json_quick["card_type"] = "quick_reply"
    response_json_quick["header"] = ""
    response_json_quick["body"] = response["title"]
    response_json_quick["caption"] = ""
    filter_buttons = ['Invalid-choice-message','prompt','Card_Type','List_Header','List_Category']
    response["options"] = [x for x in response['options'] if x not in filter_buttons]
    response_json_quick["options"] = response["options"]
#     print(response_json_quick)
    show_response_card(gs_context, current_context, response_json_quick)


def format_response_card_list(gs_context, current_context, response, temp_menu):
    response_json_list = {}
    # print(temp_menu)
    response_json_list['card_type'] = temp_menu['Card_Type']
    response_json_list['main_title'] = temp_menu['List_Header']
    response_json_list['main_body'] = response['title']
    response_json_list['button_text'] = "Show list"
    response_json_list["sections"] = []
    response_json_list_cat1 = {}
    response_json_list_cat1["title"] = temp_menu['List_category']
    options = []
    filter_buttons = ['Invalid-choice-message','prompt','Card_Type','List_Header','List_category']
    response["options"] = [x for x in response['options'] if x not in filter_buttons]
    for button in response["options"]:
        each_option = {}
        each_option["title"] = button
        options.append(each_option)
    response_json_list_cat1["options"] = options
    response_json_list["sections"].append(response_json_list_cat1)
    show_response_card(gs_context, current_context, response_json_list)
#     print(response_json_list)


def activate_menu(gs_context, current_context, selection):
    '''
        This function activates the handle active menu until
        the menu_handled is true.
    '''
    current_context['current_intent']['user_text'] = selection
    gs_context['active_menu'] = selection
    # gs_context['active_menu_state'].append(selection)
    gs_context['menu_handled'] = True
    handle_active_menu(gs_context, current_context)
    return


def handle_active_menu(gs_context,current_context):
    """
        This function will come handy when it is a live dictionary
    """
#     selection = current_context['current_intent']['user_text']
#     print('Print whether the live dictionary is list',isinstance(gs_context['live_dict'][selection],dict))
    if gs_context['menu_handled']:
        selection = current_context['current_intent']['user_text']
        temp_selection = selection.split(" ")
        if len(temp_selection) >1 and temp_selection[-1].isdigit():
            selection = " ".join(temp_selection[:-1])
        response ={}
        if isinstance(gs_context['live_dict'],dict):
            available_key_live_dict = list(gs_context['live_dict'].keys())
        elif isinstance(gs_context['live_dict'],list):
            available_key_live_dict = gs_context['live_dict']
        try:
            if selection in available_key_live_dict:
                temp_menu = gs_context['live_dict'][selection].copy()
                try:
                    option_to_user = list(temp_menu.keys())#available dictionary
                    response['title'] = temp_menu['prompt']
                except:
                    option_to_user = temp_menu
                    response['title'] = 'Please select the options'
                gs_context['menu_handled'] = True
                option_to_user = [x for x in option_to_user if x not in ['Invalid-choice-message','prompt']]
                # response['title'] = temp_menu['prompt']
                if 'Category' not in option_to_user:
                    response['options'] = option_to_user
                    gs_context['live_dict'] = temp_menu
                    print('options without Category')
                    gs_context['active_menu'] = selection
                    gs_context['active_menu_state'].append(selection)
                    # print('normal',response)
                else:
                    category_options = temp_menu['Category']
                    print('options with Category')
                    other_option = temp_menu.keys()
                    left_options = [x for x in other_option if x not in ['Category']]
                    temp_temp_menu = {}
                    for key,value in temp_menu.items():
                        if key in left_options:
                            temp_temp_menu[key] = value
                    for keys in category_options:
                        temp_temp_menu[keys] = ''
                    # print(temp_temp_menu)
                    gs_context['live_dict'] = temp_temp_menu
                    gs_context['active_menu'] = selection
                    gs_context['active_menu_state'].append(selection)
                    left_options.extend(category_options)
                    response['options'] = left_options
                    # print(left_options)
            else:
                temp_menu = gs_context['live_dict'].copy()
                print('Wrong option else')
                response['title'] = gs_context['live_dict']['Invalid-choice-message']
                option_to_user = gs_context['live_dict'].keys()
                option_to_user = [x for x in option_to_user if x not in ['Invalid-choice-message','prompt']]
                response['options'] = option_to_user
            if temp_menu['Card_Type'] == 'list':
                format_response_card_list(gs_context,current_context,response,temp_menu)
            else:
                format_response_card_quick_reply(gs_context,current_context,response)
        except Exception as e:
            if selection in gs_context['live_dict']:
                print('Here I am in the except command')
                gs_context['active_menu_state'].append(selection)
                gs_context['menu_handled']=False
                current_context["current_intent"]={'intent': 'Menu_Completed', 'entities': {}, 'user_text': ''}
            else:
                print('Thas is a wrong option please choose the one from the below')
                response['title'] = 'Thas is a wrong option please choose the one from the below'
                option_to_user = gs_context['live_dict']
                gs_context['menu_handled']=True
                response['options'] = option_to_user
                temp_menu = option_to_user
                if temp_menu['Card_Type'] == 'list':
                    format_response_card_list(gs_context,current_context,response,temp_menu)
                else:
                    format_response_card_quick_reply(gs_context,current_context,response)
    #             print(option_to_user)
                return
        return



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gs_conv_help",
    version="0.0.1",
    author="gupshup_gip",
    author_email="rahul.meka@gupshup.io",
    description="This repository contains artifacts to help accelerate development of conversational interfaces using Rasa & Gip",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
   'pymongo>=3.0',
   'pandas>=1.0',
   "pyyaml>=5.0"
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires="<3.9",
)